@@ -429,7 +429,7 @@ measure_heading (GtkWidget      *widget,
 {
   AdwAlertDialog *self = g_object_get_data (G_OBJECT (widget), "-adw-alert-dialog");
   AdwAlertDialogPrivate *priv = adw_alert_dialog_get_instance_private (self);
-  int large_min, large_nat, small_min, small_nat;
+  int large_min, large_nat, small_min, small_nat, for_size_large_label;
 
   if (priv->is_short) {
     gtk_widget_measure (priv->heading_label_small, orientation, for_size,
@@ -443,7 +443,14 @@ measure_heading (GtkWidget      *widget,
     return;
   }
 
-  gtk_widget_measure (priv->heading_label, orientation, for_size,
+  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+    gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                        &for_size_large_label, NULL, NULL, NULL);
+  } else {
+    for_size_large_label = -1;
+  }
+
+  gtk_widget_measure (priv->heading_label, orientation, for_size_large_label,
                       &large_min, &large_nat, NULL, NULL);
   gtk_widget_measure (priv->heading_label_small, orientation, for_size,
                       &small_min, &small_nat, NULL, NULL);
@@ -979,10 +986,13 @@ adw_alert_dialog_class_init (AdwAlertDialogClass *klass)
    *
    * The response ID of the default response.
    *
-   * If set, pressing <kbd>Enter</kbd> will activate the corresponding button.
+   * The button corresponding to this response will be set as the default widget
+   * of the dialog.
    *
-   * If set to `NULL` or a non-existent response ID, pressing <kbd>Enter</kbd>
-   * will do nothing.
+   * If not set, the default widget will not be set, and the last added response
+   * will be focused by default.
+   *
+   * See [property@Dialog:default-widget].
    *
    * Since: 1.5
    */
@@ -2316,10 +2326,13 @@ adw_alert_dialog_get_default_response (AdwAlertDialog *self)
  *
  * Sets the ID of the default response of @self.
  *
- * If set, pressing <kbd>Enter</kbd> will activate the corresponding button.
+ * The button corresponding to this response will be set as the default widget
+ * of @self.
  *
- * If set to `NULL` or to a non-existent response ID, pressing <kbd>Enter</kbd>
- * will do nothing.
+ * If not set, the default widget will not be set, and the last added response
+ * will be focused by default.
+ *
+ * See [property@Dialog:default-widget].
  *
  * Since: 1.5
  */
