@@ -27,7 +27,7 @@
  * ## AdwExpanderRow as GtkBuildable
  *
  * The `AdwExpanderRow` implementation of the [iface@Gtk.Buildable] interface
- * supports adding a child as an action widget by specifying “action” as the
+ * supports adding a child as an suffix widget by specifying “suffix” as the
  * “type” attribute of a <child> element.
  *
  * It also supports adding it as a prefix widget by specifying “prefix” as the
@@ -41,12 +41,21 @@
  * It contains the subnodes `row.header` for its main embedded row,
  * `list.nested` for the list it can expand, and `image.expander-row-arrow` for
  * its arrow.
+ *
+ * ## Style classes
+ *
+ * `AdwExpanderRow` can use the [`.`](style-classes.html#property-rows)
+ * style class to emphasize the row subtitle instead of the row title, which is
+ * useful for displaying read-only properties.
+ *
+ * When used together with the `.monospace` style class, only the subtitle
+ * becomes monospace, not the title or any extra widgets.
  */
 
 typedef struct
 {
   GtkBox *box;
-  GtkBox *actions;
+  GtkBox *suffixes;
   GtkBox *prefixes;
   GtkListBox *list;
   AdwActionRow *action_row;
@@ -105,6 +114,15 @@ keynav_failed_cb (AdwExpanderRow   *self,
                                  GTK_DIR_TAB_BACKWARD : GTK_DIR_TAB_FORWARD);
 }
 
+static gboolean
+adw_expander_row_grab_focus (GtkWidget *widget)
+{
+  AdwExpanderRow *self = ADW_EXPANDER_ROW (widget);
+  AdwExpanderRowPrivate *priv = adw_expander_row_get_instance_private (self);
+
+  return gtk_widget_grab_focus (GTK_WIDGET (priv->action_row));
+}
+
 static void
 adw_expander_row_get_property (GObject    *object,
                                guint       prop_id,
@@ -185,10 +203,10 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
   object_class->set_property = adw_expander_row_set_property;
 
   widget_class->focus = adw_widget_focus_child;
-  widget_class->grab_focus = adw_widget_grab_focus_child;
+  widget_class->grab_focus = adw_expander_row_grab_focus;
 
   /**
-   * AdwExpanderRow:subtitle: (attributes org.gtk.Property.get=adw_expander_row_get_subtitle org.gtk.Property.set=adw_expander_row_set_subtitle)
+   * AdwExpanderRow:subtitle:
    *
    * The subtitle for this row.
    *
@@ -201,7 +219,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:icon-name: (attributes org.gtk.Property.get=adw_expander_row_get_icon_name org.gtk.Property.set=adw_expander_row_set_icon_name)
+   * AdwExpanderRow:icon-name:
    *
    * The icon name for this row.
    *
@@ -213,7 +231,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:expanded: (attributes org.gtk.Property.get=adw_expander_row_get_expanded org.gtk.Property.set=adw_expander_row_set_expanded)
+   * AdwExpanderRow:expanded:
    *
    * Whether the row is expanded.
    */
@@ -223,7 +241,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:enable-expansion: (attributes org.gtk.Property.get=adw_expander_row_get_enable_expansion org.gtk.Property.set=adw_expander_row_set_enable_expansion)
+   * AdwExpanderRow:enable-expansion:
    *
    * Whether expansion is enabled.
    */
@@ -233,7 +251,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:show-enable-switch: (attributes org.gtk.Property.get=adw_expander_row_get_show_enable_switch org.gtk.Property.set=adw_expander_row_set_show_enable_switch)
+   * AdwExpanderRow:show-enable-switch:
    *
    * Whether the switch enabling the expansion is visible.
    */
@@ -243,7 +261,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:title-lines: (attributes org.gtk.Property.get=adw_expander_row_get_title_lines org.gtk.Property.set=adw_expander_row_set_title_lines)
+   * AdwExpanderRow:title-lines:
    *
    * The number of lines at the end of which the title label will be ellipsized.
    *
@@ -258,7 +276,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwExpanderRow:subtitle-lines: (attributes org.gtk.Property.get=adw_expander_row_get_subtitle_lines org.gtk.Property.set=adw_expander_row_set_subtitle_lines)
+   * AdwExpanderRow:subtitle-lines:
    *
    * The number of lines at the end of which the subtitle label will be
    * ellipsized.
@@ -279,7 +297,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
                                                "/org/gnome/Adwaita/ui/adw-expander-row.ui");
   gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, action_row);
   gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, box);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, actions);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, suffixes);
   gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, list);
   gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, image);
   gtk_widget_class_bind_template_child_private (widget_class, AdwExpanderRow, enable_switch);
@@ -328,7 +346,9 @@ adw_expander_row_buildable_add_child (GtkBuildable *buildable,
   if (!priv->box)
     parent_buildable_iface->add_child (buildable, builder, child, type);
   else if (type && strcmp (type, "action") == 0)
-    adw_expander_row_add_action (self, GTK_WIDGET (child));
+    adw_expander_row_add_suffix (self, GTK_WIDGET (child));
+  else if (type && strcmp (type, "suffix") == 0)
+    adw_expander_row_add_suffix (self, GTK_WIDGET (child));
   else if (type && strcmp (type, "prefix") == 0)
     adw_expander_row_add_prefix (self, GTK_WIDGET (child));
   else if (!type && GTK_IS_WIDGET (child))
@@ -364,6 +384,8 @@ adw_expander_row_new (void)
  * @widget: a widget
  *
  * Adds an action widget to @self.
+ *
+ * Deprecated: 1.4: Use [method@ExpanderRow.add_suffix] to add a suffix.
  */
 void
 adw_expander_row_add_action (AdwExpanderRow *self,
@@ -373,11 +395,12 @@ adw_expander_row_add_action (AdwExpanderRow *self,
 
   g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (self));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_expander_row_get_instance_private (self);
 
-  gtk_box_prepend (priv->actions, widget);
-  gtk_widget_set_visible (GTK_WIDGET (priv->actions), TRUE);
+  gtk_box_prepend (priv->suffixes, widget);
+  gtk_widget_set_visible (GTK_WIDGET (priv->suffixes), TRUE);
 }
 
 /**
@@ -395,6 +418,7 @@ adw_expander_row_add_prefix (AdwExpanderRow *self,
 
   g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_expander_row_get_instance_private (self);
 
@@ -405,6 +429,31 @@ adw_expander_row_add_prefix (AdwExpanderRow *self,
   gtk_box_append (priv->prefixes, widget);
 }
 
+/**
+ * adw_expander_row_add_suffix:
+ * @self: an expander row
+ * @widget: a widget
+ *
+ * Adds an suffix widget to @self.
+ *
+ * Since: 1.4
+ */
+void
+adw_expander_row_add_suffix (AdwExpanderRow *self,
+                             GtkWidget      *widget)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (self));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  gtk_box_prepend (priv->suffixes, widget);
+  gtk_widget_set_visible (GTK_WIDGET (priv->suffixes), TRUE);
+}
+
 /**
  * adw_expander_row_add_row:
  * @self: an expander row
@@ -422,6 +471,7 @@ adw_expander_row_add_row (AdwExpanderRow *self,
 
   g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
   priv = adw_expander_row_get_instance_private (self);
 
@@ -434,9 +484,9 @@ adw_expander_row_add_row (AdwExpanderRow *self,
 }
 
 /**
- * adw_action_row_expander:
+ * adw_expander_row_remove:
  * @self: an expander row
- * @widget: the child to be removed
+ * @child: the child to be removed
  *
  * Removes a child from @self.
  */
@@ -454,7 +504,7 @@ adw_expander_row_remove (AdwExpanderRow *self,
 
   parent = gtk_widget_get_parent (child);
 
-  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->actions)) {
+  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->suffixes)) {
     gtk_box_remove (GTK_BOX (parent), child);
     gtk_widget_set_visible (parent, gtk_widget_get_first_child (parent) != NULL);
   }
@@ -471,7 +521,7 @@ adw_expander_row_remove (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
+ * adw_expander_row_get_subtitle:
  * @self: an expander row
  *
  * Gets the subtitle for @self.
@@ -491,7 +541,7 @@ adw_expander_row_get_subtitle (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_subtitle: (attributes org.gtk.Method.set_property=subtitle)
+ * adw_expander_row_set_subtitle:
  * @self: an expander row
  * @subtitle: the subtitle
  *
@@ -514,7 +564,7 @@ adw_expander_row_set_subtitle (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_expander_row_get_icon_name:
  * @self: an expander row
  *
  * Gets the icon name for @self.
@@ -536,7 +586,7 @@ adw_expander_row_get_icon_name (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_expander_row_set_icon_name:
  * @self: an expander row
  * @icon_name: (nullable): the icon name
  *
@@ -558,7 +608,7 @@ adw_expander_row_set_icon_name (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_expanded: (attributes org.gtk.Method.get_property=expanded)
+ * adw_expander_row_get_expanded:
  * @self: an expander row
  *
  * Gets whether @self is expanded.
@@ -578,7 +628,7 @@ adw_expander_row_get_expanded (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_expanded: (attributes org.gtk.Method.set_property=expanded)
+ * adw_expander_row_set_expanded:
  * @self: an expander row
  * @expanded: whether to expand the row
  *
@@ -614,7 +664,7 @@ adw_expander_row_set_expanded (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_enable_expansion: (attributes org.gtk.Method.get_property=enable-expansion)
+ * adw_expander_row_get_enable_expansion:
  * @self: an expander row
  *
  * Gets whether the expansion of @self is enabled.
@@ -634,7 +684,7 @@ adw_expander_row_get_enable_expansion (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_enable_expansion: (attributes org.gtk.Method.set_property=enable-expansion)
+ * adw_expander_row_set_enable_expansion:
  * @self: an expander row
  * @enable_expansion: whether to enable the expansion
  *
@@ -663,7 +713,7 @@ adw_expander_row_set_enable_expansion (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_show_enable_switch: (attributes org.gtk.Method.get_property=show-enable-switch)
+ * adw_expander_row_get_show_enable_switch:
  * @self: an expander row
  *
  * Gets whether the switch enabling the expansion of @self is visible.
@@ -683,7 +733,7 @@ adw_expander_row_get_show_enable_switch (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_show_enable_switch: (attributes org.gtk.Method.set_property=show-enable-switch)
+ * adw_expander_row_set_show_enable_switch:
  * @self: an expander row
  * @show_enable_switch: whether to show the switch enabling the expansion
  *
@@ -710,7 +760,7 @@ adw_expander_row_set_show_enable_switch (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
+ * adw_expander_row_get_title_lines:
  * @self: an expander row
  *
  * Gets the number of lines at the end of which the title label will be
@@ -734,7 +784,7 @@ adw_expander_row_get_title_lines (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
+ * adw_expander_row_set_title_lines:
  * @self: an expander row
  * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
@@ -759,7 +809,7 @@ adw_expander_row_set_title_lines (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
+ * adw_expander_row_get_subtitle_lines:
  * @self: an expander row
  *
  * Gets the number of lines at the end of which the subtitle label will be
@@ -783,7 +833,7 @@ adw_expander_row_get_subtitle_lines (AdwExpanderRow *self)
 }
 
 /**
- * adw_expander_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
+ * adw_expander_row_set_subtitle_lines:
  * @self: an expander row
  * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
