@@ -406,22 +406,29 @@ parent_changed_cb (AdwMessageDialog *self)
   set_parent (self, transient_for);
 }
 
-static void
-button_clicked_cb (ResponseInfo *info)
+
+static inline void
+emit_response (AdwMessageDialog *self, GQuark response)
 {
-  AdwMessageDialog *self = info->dialog;
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
 
   g_object_ref (self);
   priv->block_close_response = TRUE;
 
   gtk_window_close (GTK_WINDOW (self));
-  g_signal_emit (self, signals[SIGNAL_RESPONSE], info->id, g_quark_to_string (info->id));
+  g_signal_emit (self, signals[SIGNAL_RESPONSE], response, g_quark_to_string (response));
 
   priv->block_close_response = FALSE;
   g_object_unref (self);
 }
 
+
+static void
+button_clicked_cb (ResponseInfo *info)
+{
+  emit_response (info->dialog, info->id);
+}
+
 static GtkWidget *
 create_response_button (AdwMessageDialog *self,
                         ResponseInfo     *info)
@@ -555,7 +562,7 @@ measure_heading (GtkWidget      *widget,
 {
   AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
-  int large_min, large_nat, small_min, small_nat;
+  int large_min, large_nat, small_min, small_nat, for_size_large_label;
 
   if (gtk_widget_has_css_class (GTK_WIDGET (self), "short")) {
     gtk_widget_measure (priv->heading_label_small, orientation, for_size,
@@ -569,7 +576,14 @@ measure_heading (GtkWidget      *widget,
     return;
   }
 
-  gtk_widget_measure (priv->heading_label, orientation, for_size,
+  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+    gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                        &for_size_large_label, NULL, NULL, NULL);
+  } else {
+    for_size_large_label = -1;
+  }
+
+  gtk_widget_measure (priv->heading_label, orientation, for_size_large_label,
                       &large_min, &large_nat, NULL, NULL);
   gtk_widget_measure (priv->heading_label_small, orientation, for_size,
                       &small_min, &small_nat, NULL, NULL);
@@ -1072,10 +1086,13 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    *
    * The response ID of the default response.
    *
-   * If set, pressing <kbd>Enter</kbd> will activate the corresponding button.
+   * The button corresponding to this response will be set as the default widget
+   * of the dialog.
    *
-   * If set to `NULL` or a non-existent response ID, pressing <kbd>Enter</kbd>
-   * will do nothing.
+   * If not set, the default widget will not be set, and the last added response
+   * will be focused by default.
+   *
+   * See [property@Gtk.Window:default-widget].
    *
    * Since: 1.2
    * Deprecated: 1.6: Use [class@AlertDialog].
@@ -2377,10 +2394,13 @@ adw_message_dialog_get_default_response (AdwMessageDialog *self)
  *
  * Sets the ID of the default response of @self.
  *
- * If set, pressing <kbd>Enter</kbd> will activate the corresponding button.
+ * The button corresponding to this response will be set as the default widget
+ * of @self.
  *
- * If set to `NULL` or to a non-existent response ID, pressing <kbd>Enter</kbd>
- * will do nothing.
+ * If not set, the default widget will not be set, and the last added response
+ * will be focused by default.
+ *
+ * See [property@Gtk.Window:default-widget].
  *
  * Since: 1.2
  * Deprecated: 1.6: Use [class@AlertDialog].
@@ -2542,8 +2562,9 @@ choose_cancelled_cb (GCancellable *cancellable,
                      GTask        *task)
 {
   AdwMessageDialog *self = g_task_get_source_object (task);
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
 
-  choose_response_cb (self, adw_message_dialog_get_close_response (self), task);
+  emit_response (self, priv->close_response);
 }
 
 /**
